package category

import (
	"context"
	"errors"
	"fmt"

	"git.3crabs.ru/save_my_money/smm_core/internal/services/context_utils"
	"github.com/jackc/pgx/v5"
	"github.com/jackc/pgx/v5/pgconn"
	"github.com/jackc/pgx/v5/pgxpool"
)

type CategoryEntity struct {
	Id           int
	Name         string
	UserId       int
	Favorite     bool
	MonthlyLimit int
}

type CategoryAlreadyExistsErr struct{}

func (e *CategoryAlreadyExistsErr) Error() string {
	return "category already exists error"
}

type CategoryService struct {
	db *pgxpool.Pool
}

func NewCategoryService(
	db *pgxpool.Pool,
) *CategoryService {
	return &CategoryService{
		db: db,
	}
}

func (s *CategoryService) AddCategory(ctx context.Context, category *CategoryEntity) (*CategoryEntity, error) {
	userId, err := context_utils.GetUserId(ctx)
	if err != nil {
		return nil, err
	}
	query := `INSERT INTO categories (name, user_id, favorite, monthly_limit) VALUES (@name, @user_id, @favorite, @monthly_limit) RETURNING id`
	args := pgx.NamedArgs{
		"name":          category.Name,
		"user_id":       userId,
		"favorite":      category.Favorite,
		"monthly_limit": category.MonthlyLimit,
	}
	if err := s.db.QueryRow(ctx, query, args).Scan(&category.Id); err != nil {
		var pgErr *pgconn.PgError
		if errors.As(err, &pgErr) {
			if pgErr.Code == "23505" && pgErr.ConstraintName == "categories_user_id_name_key" { // unique_violation
				return nil, &CategoryAlreadyExistsErr{}
			}
		}
		return nil, fmt.Errorf("unable to insert row: %w", err)
	}
	return category, nil
}
